---
author: Martin Weise
---

# User Interface

## tl;dr

!!! debug "Debug Information"

    Image: [`dbrepo/ui:__APPVERSION__`](https://hub.docker.com/r/dbrepo/ui)

    * Ports: 3000/tcp, 9100/tcp
    * Prometheus: `http://<hostname>:9100/metrics`
    * UI: `http://<hostname>/`

## Overview

It provides a graphical interface for a researcher to interact with the API (c.f. Figure 1). 

<figure markdown>
![User Interface](images/screenshots/ui.png){ .img-border }
<figcaption>Figure 1: User Interface</figcaption>
</figure>

For examples on how to use the User Interface, visit the [Usage Overview](../usage-overview/) to find out how to create
users, databases and how to import your data.

### Settings

The User Interface can be configured extensively with 
the [`dbrepo.config.json`](https://gitlab.phaidra.org/fair-data-austria-db-repository/fda-services/-/raw/master/dbrepo-ui/dbrepo.config.json)
configuration file, mounted directly into the container with e.g. docker compose. As a small example, you can configure
the logo :material-numeric-1-circle-outline: in Figure 2. Make sure you mount the logo as image as well, in this example
we want to mount a custom logo `my_logo.png` into the container and specify the name.

<figure markdown>
![Architecture of the UI microservice](images/screenshots/ui-config-step-1.png){ .img-border }
<figcaption>Figure 2: Architecture of the UI microservice</figcaption>
</figure>

Text values like the version :material-numeric-2-circle-outline: and title :material-numeric-3-circle-outline: can be
configured as well via the `dbrepo.config.json` values file. The important links section 
:material-numeric-4-circle-outline: can be modified or removed entirely by setting `page.information.links` to `[]`.

```json title="dbrepo.config.json"
{
    "title": "Database Repository",
    "version": "__APPVERSION___DOCKER-COMPOSE",
    "logo": {
      "path": "/my_logo.png"
    },
    "page": {
    "information": {
      "links": []
    },
    ...
}
```

To work, you need to mount the `my_logo.png` file into the `dbrepo-ui` container via the `docker-compose.yml` file (or
if you use a Kubernetes deployment via ConfigMap and Volumes).

```yaml title="docker-compose.yml"
services:
  dbrepo-ui:
    image: docker.io/dbrepo/ui:__APPVERSION__
    volumes:
      - ./my_logo.png:/app/static/my_logo.png
      - ./dbrepo.conf.json:/app/dbrepo.conf.json
  ...
```

### Architecture

The server-client architecture of the User Interface is shown in [Figure 3](#fig3), it is supposed to help debug the
User Interface on development.

<figure id="fig3" markdown>
![Architecture of the UI microservice](images/architecture-ui.svg)
<figcaption>Figure 3: Architecture of the User Interface</figcaption>
</figure>

### Example

See the [Usage Overview](../usage-overview/) page for detailed examples.

## Limitations

(none)

!!! question "Do you miss functionality? Do these limitations affect you?"

    We strongly encourage you to help us implement it as we are welcoming contributors to open-source software and get
    in [contact](../contact) with us, we happily answer requests for collaboration with attached CV and your programming 
    experience!

## Security

(none)
