package at.tuwien.api.database.table.columns;

import at.tuwien.api.container.image.ImageDateDto;
import at.tuwien.api.database.table.columns.concepts.ConceptDto;
import at.tuwien.api.database.table.columns.concepts.UnitDto;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import lombok.*;
import lombok.extern.jackson.Jacksonized;
import org.springframework.data.annotation.Id;
import org.springframework.data.elasticsearch.annotations.Field;
import org.springframework.data.elasticsearch.annotations.FieldType;

import java.math.BigDecimal;
import java.util.List;

@Getter
@Setter
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Jacksonized
@ToString
public class ColumnDto {

    @Id
    @NotNull
    @Field(name = "id", type = FieldType.Keyword)
    private Long id;

    @NotNull
    @Field(name = "database_id", type = FieldType.Keyword)
    @JsonProperty("database_id")
    private Long databaseId;

    @NotNull
    @Field(name = "table_id", type = FieldType.Keyword)
    @JsonProperty("table_id")
    private Long tableId;

    @NotBlank
    @Schema(example = "Date")
    @Field(name = "name", type = FieldType.Keyword)
    private String name;

    @NotBlank
    @JsonProperty("internal_name")
    @Field(name = "internal_name", type = FieldType.Keyword)
    @Schema(example = "mdb_date")
    private String internalName;

    @Field(name = "alias", type = FieldType.Keyword)
    @Schema
    private String alias;

    @JsonProperty("date_format")
    @Field(name = "date_format", type = FieldType.Object)
    private ImageDateDto dateFormat;

    @NotNull
    @JsonProperty("auto_generated")
    @Field(name = "auto_generated", type = FieldType.Boolean)
    @Schema(example = "false")
    private Boolean autoGenerated;

    @NotNull
    @JsonProperty("is_primary_key")
    @Field(name = "is_primary_key", type = FieldType.Boolean)
    @Schema(example = "true")
    private Boolean isPrimaryKey;

    @Field(name = "index_length", type = FieldType.Long)
    @JsonProperty("index_length")
    private Long indexLength;

    @Field(name = "length", type = FieldType.Long)
    @JsonProperty("length")
    private Long length;

    @NotNull
    @JsonProperty("column_type")
    @Field(name = "column_type", type = FieldType.Keyword)
    @Schema(example = "string")
    private ColumnTypeDto columnType;

    @Schema(example = "255")
    @Field(name = "size", type = FieldType.Long)
    private Long size;

    @Schema(example = "0")
    @Field(name = "d", type = FieldType.Long)
    private Long d;

    @Schema(example = "34300")
    @Field(name = "data_length", type = FieldType.Long)
    private Long dataLength;

    @Schema(example = "34300")
    @Field(name = "max_data_length", type = FieldType.Long)
    private Long maxDataLength;

    @Schema(example = "32")
    @Field(name = "num_rows", type = FieldType.Long)
    private Long numRows;

    @Schema(example = "0")
    @Field(name = "val_min", type = FieldType.Double)
    private BigDecimal valMin;

    @Schema(example = "100")
    @Field(name = "val_max", type = FieldType.Double)
    private BigDecimal valMax;

    @Schema(example = "45.4")
    @Field(name = "mean", type = FieldType.Double)
    private BigDecimal mean;

    @Schema(example = "51")
    @Field(name = "median", type = FieldType.Double)
    private BigDecimal median;

    @Schema(example = "5.32")
    @Field(name = "std_dev", type = FieldType.Double)
    private BigDecimal stdDev;

    @Field(name = "concept", type = FieldType.Object)
    private ConceptDto concept;

    @Field(name = "unit", type = FieldType.Object)
    private UnitDto unit;

    @NotNull
    @JsonProperty("is_public")
    @Field(name = "is_public", type = FieldType.Boolean)
    @Schema(example = "true")
    private Boolean isPublic;

    @NotNull
    @JsonProperty("is_null_allowed")
    @Field(name = "is_null_allowed", type = FieldType.Boolean)
    @Schema(example = "false")
    private Boolean isNullAllowed;

    @Parameter(description = "enum values, only considered when type = ENUM")
    private List<String> enums;

    @Parameter(description = "enum values, only considered when type = ENUM")
    private List<String> sets;

}
